/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CWindow.cpp  -  description
                             -------------------
    begin                : Sun May 18 2003
    copyright            : (C) 2000-2006 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <sptk3/cxml>
#include <sptk3/CWindow.h>
#include <FL/Fl.H>

using namespace std;
using namespace sptk;

void CWindow::ctor_init(const char *label) {
   if (label)
      m_label = label;
   Fl_Double_Window::label(m_label.c_str());
   CLayoutManager::attach(this); 
}

void CWindow::resize(int xx,int yy,int ww,int hh) {
   int www = ww, hhh = hh;
   // For a window, layout is always located at 0,0 coordinates
   autoLayout(0,0,www,hhh,true);
   if (www > Fl::w()) www = Fl::w();
   if (hhh > Fl::h()) www = Fl::h();
   if (xx + www > Fl::w()) xx = Fl::w() - www;
   if (yy + hhh > Fl::h()) yy = Fl::h() - hhh;
   Fl_Double_Window::resize(xx,yy,www,hhh);
}

void CWindow::preferredSize(int& ww,int& hh) {
   autoLayout(0,0,ww,hh,false);
}

void CWindow::draw() {
   int savex = x(); x(0);
   int savey = y(); y(0);

   Fl_Boxtype b = box();
   if (damage() & ~FL_DAMAGE_CHILD) { // redraw the entire thing:
      draw_box();
      paintBackground();
   }

   box(FL_NO_BOX);
   Fl_Double_Window::draw();
   box(b);

   y(savey);
   x(savex);
}

void CWindow::load(const CXmlNode& node) {
   CGroupXml::load(node);
   loadPosition(node);
}

void CWindow::save(CXmlNode& node) const {
   CGroupXml::save(node);
   savePosition(node);
}

void CWindow::loadPosition(const CXmlNode& node) {
   int hh = node.getAttribute("height");
   int ww = node.getAttribute("width");
   if (hh > 0 && ww > 0) 
      resize(node.getAttribute("x","0"),node.getAttribute("y","0"),ww,hh);
}

void CWindow::savePosition(CXmlNode& node) const {
   node.attributes["x"] = x();
   node.attributes["y"] = y();
   node.attributes["width"] = w();
   node.attributes["height"] = h();
   node.name("window");
}
